package versionedconsumer

import (
	"log"
	"os"
	"testing"

	"github.com/Shopify/sarama"
	"github.com/stretchr/testify/assert"
)

func TestVersionedConsumer(t *testing.T) {

	testcases := []struct {
		name    string
		version sarama.KafkaVersion
	}{
		{"0.9.0.1", sarama.V0_9_0_1},
		{"0.10.1.0", sarama.V0_10_1_0},
	}

	for _, tc := range testcases {
		t.Run(tc.name, func(t *testing.T) {
			testConsumer(t, tc.version)
		})
	}
}

func testConsumer(t *testing.T, version sarama.KafkaVersion) {
	sarama.Logger = log.New(os.Stdout, "[sarama] ", log.LstdFlags)
	topic := "my_topic"
	partition := int32(0)
	broker := sarama.NewMockBroker(t, 0)
	defer broker.Close()
	broker.SetHandlerByMap(map[string]sarama.MockResponse{
		"MetadataRequest": sarama.NewMockMetadataResponse(t).
			SetBroker(broker.Addr(), broker.BrokerID()).
			SetLeader(topic, partition, broker.BrokerID()),
		"OffsetRequest": sarama.NewMockOffsetResponse(t).
			SetOffset(topic, partition, sarama.OffsetOldest, 0).
			SetOffset(topic, partition, sarama.OffsetNewest, 1000),
		"FetchRequest": sarama.NewMockFetchResponse(t, 1).
			SetMessage(topic, partition, 0, sarama.StringEncoder("HELLO")),
	})
	consumerConfig := sarama.NewConfig()
	consumerConfig.Version = version
	consumer, err := sarama.NewConsumer([]string{broker.Addr()}, consumerConfig)
	if err != nil {
		t.Fatal(err)
	}
	partitionConsumer, err := consumer.ConsumePartition(topic, partition, sarama.OffsetOldest)
	if err != nil {
		t.Fatal(err)
	}
	message := <-partitionConsumer.Messages()
	assert.EqualValues(t, []byte("HELLO"), message.Value)

}
